/*
Copyright (c) 2001 David Drysdale

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the author shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the author.
*/

import java.applet.*;
import java.awt.*;
import java.net.URL;
import java.io.InputStream;


public class GridFont extends Applet {
    public boolean debugMode = false;

    // size of each grid letter
    public int numHDivs = 2;
    public int numVDivs = 6;
    
    // whether we are currently displaying the grid
    boolean gridOn = true;

    // Various buttons
    Button loadButton;
    Button saveButton;
    Button clearAllButton;
    Button gridButton;

    Button clearButton;
    Button copyButton;
    Button pasteButton;
    Button hflipButton;
    Button vflipButton;
    Button rotateButton;

    Button downloadButton = null;
    InputStream loadStream = null;
    GridCanvas gridCanvas;
    TextField nameField;

    Component theFrame;
    
    // the database of pre-existing fonts
    String fontDB = null; // all in one big string
    int numFonts;
    FontDescription[] fonts;

    public boolean showGrid() {return gridOn;}

    public String[][] getParameterInfo() {
        String[][] info = {
            {"norows", "integer", "number of rows"},
            {"nocols", "integer", "number of columns"},
            {"debug",  "true false", "whether to display debug info"}
        };
        return info;
    }

    public String getAppletInfo() {
        return "GridFont v1.0 (c) David Drysdale 2001";
    }

    public void init() {
        if (debugMode) System.out.println("Init applet");
        // read in parameters
        int row = 0;
        boolean failed = false;
        String s = this.getParameter("norows");
        try { row = Integer.parseInt(s); }
        catch (NumberFormatException e) { failed = true;}
        if ((!failed) && (row > 0)) {
          numVDivs = row;
        }
        int col = 0;
        failed = false;
        s = this.getParameter("nocols");
        try { col = Integer.parseInt(s); }
        catch (NumberFormatException e) { failed = true;}
        if ((!failed) && (col > 0)) {
          numHDivs = col;
        }
        Boolean dm = new Boolean(this.getParameter("debug"));
        debugMode = dm.booleanValue();

        // find an owning frame
        if (debugMode) System.out.println("Find owning frame");
        theFrame = getParent();
        while ((theFrame != null) && !(theFrame instanceof Frame)) {
            theFrame = theFrame.getParent();
        }
        setLayout(new BorderLayout());

        // Set up the layout of the buttons at the top
        Panel buttonPanel = new Panel();

        // First see if we can find the database of gridfonts on the server. If so,
        // add the load button
        if (debugMode) System.out.println("Look for DB");
        boolean foundDB = false;
        try {
            URL db = new URL(getDocumentBase(), "gridfonts.data");
            try { 
                loadStream = db.openStream();
                foundDB = true;
            } catch (java.io.IOException e) {
                if (debugMode) System.out.println("Failed to open stream " + e);
            } catch (SecurityException se) {
                if (debugMode) System.out.println("Security exception " + se);
            }
        } catch (java.net.MalformedURLException e) {
            if (debugMode) System.out.println("Malformed URL" + e);
        }
        if (foundDB) {
            downloadButton = new Button("Download...");
            buttonPanel.add(downloadButton);
        }

        loadButton = new Button("Load");
        buttonPanel.add(loadButton);
        saveButton = new Button("Save");
        buttonPanel.add(saveButton);
        gridButton = new Button("Remove Grid");
        buttonPanel.add(gridButton);
        clearAllButton = new Button("Clear All");
        buttonPanel.add(clearAllButton);

        buttonPanel.add(new Label(""));

        clearButton = new Button("Clear");
        buttonPanel.add(clearButton);
        copyButton = new Button("Copy");
        buttonPanel.add(copyButton);
        pasteButton = new Button("Paste");
        buttonPanel.add(pasteButton);
        vflipButton = new Button("VFlip");
        buttonPanel.add(vflipButton);
        hflipButton = new Button("HFlip");
        buttonPanel.add(hflipButton);
        rotateButton = new Button("Rotate");
        buttonPanel.add(rotateButton);
        
        Panel namePanel = new Panel();
        namePanel.add(new Label("Name of font:"));
        nameField = new TextField(15);

        namePanel.add(nameField);

        if (debugMode) System.out.println("Add top and bottom panels");
        add("North", buttonPanel);
        add("South", namePanel);

        Dimension d = size();
        if (debugMode) System.out.println("Create GridCanvas instance");
        gridCanvas = new GridCanvas(this);
        gridCanvas.resize(d.width, 
                          d.height - buttonPanel.minimumSize().height 
                          - namePanel.minimumSize().height); 
        add("Center", gridCanvas);
        
        gridCanvas.init();
        if (debugMode) System.out.println("Time to show()");
        show();
    }

    public void setFontName(String name) {
        nameField.setText(name);
    }        

    public boolean action(Event e, Object what) {
        boolean processed = true;

        if (debugMode) System.out.println("Examine event "+e+" on object "+what);
        if (e.target == loadButton) {
            LoadDialog load = new LoadDialog((Frame)theFrame, true);
            load.resize(400, 400);
            Dimension d = size();
            load.move((d.width/2) - (load.size().width/2), 
                      (d.height/2) + 100 - (load.size().height/2));
            // Note that we explicitly use the show() method of Window, not
            // Dialog.  This allows the code to work on JDK 1.02, since show()
            // only became a method of Dialog in JDK 1.1 (passim)
            ((Window)load).show();
            load.dispose();
            if (load.getAnswer()) {
                String newFont = load.getText();
                gridCanvas.setFontString(newFont);
            }
        } else if (e.target == saveButton) {
            SaveDialog save = new SaveDialog((Frame)theFrame, true, 
                                             gridCanvas.getFontString(nameField.getText()));
            save.resize(400, 400);
            Dimension d = size();
            save.move((d.width/2) - (save.size().width/2), 
                      (d.height/2) + 100 - (save.size().height/2));
            ((Window)save).show();
            save.dispose();
        } else if (e.target == clearAllButton) {
            ConfirmDialog confirm = new ConfirmDialog((Frame)theFrame, true);
            Dimension d = size();
            confirm.move(d.width/2, d.height/2);
            if (debugMode) System.out.println("put up the newly constructed confirm dialog");
            ((Window)confirm).show(); //@@@ never returns on JDK1.02/Win appletviewer
            if (debugMode) System.out.println("bring down the confirm dialog");
            confirm.dispose();
            if (confirm.getAnswer()) {
                nameField.setText(new String(""));
                gridCanvas.clearAll();
            }
        } else if (e.target == clearButton) {
            gridCanvas.clearCurrent();
        } else if (e.target == copyButton) {
            gridCanvas.copyCurrent();
        } else if (e.target == pasteButton) {
            gridCanvas.pasteCurrent();
        } else if (e.target == vflipButton) {
            gridCanvas.vflipCurrent();
        } else if (e.target == hflipButton) {
            gridCanvas.hflipCurrent();
        } else if (e.target == rotateButton) {
            gridCanvas.rotateCurrent();
        } else if (e.target == gridButton) {
            if (gridOn) {
                gridOn = false;
                gridButton.setLabel("Show Grid");
            } else {
                gridOn = true;
                gridButton.setLabel("Remove Grid");
            }
            gridCanvas.paint(gridCanvas.getGraphics());
        } else if (e.target == downloadButton) {
            // if we haven't already, slurp up the whole file into one big string
            if (fontDB == null) {
                if (loadStream != null) {
                    try {
                        StringBuffer buf = new StringBuffer();
                        int c;
                        while ((c = loadStream.read()) > -1) {
                            buf.append((char)c);
                        }
                        // prefix with "\n" so we don't have to special case below
                        fontDB = "\n" + buf.toString();

                        // Now split the file into individual fonts, looking
                        // for "font" as the delimiter.  First count the number
                        // of fonts
                        numFonts = 0;
                        int offset = fontDB.indexOf("\nfont", 0);
                        while (offset != -1) {
                            numFonts++;
                            offset++; // move past the current instance
                            offset = fontDB.indexOf("\nfont", offset);
                        }
                            
                        fonts = new FontDescription[numFonts];
                        offset = 0;
                        for (int ii=0; ii<numFonts; ii++) {
                            int start = fontDB.indexOf("\nfont", offset);
                            int end = fontDB.indexOf("\nfont",start+1);
                            if (end == -1) end = fontDB.length();
                            
                            fonts[ii] = new FontDescription(fontDB.substring(start, end));
                            offset = end;
                        }
                    } catch (java.io.IOException eio) {}
                }
            }
            if (fontDB != null) {
                if (debugMode) System.out.println("List " + fonts.length+ " fonts");
                FontSelectionDialog fld = new FontSelectionDialog((Frame)theFrame, true, fonts);
                fld.resize(400, 500);
                Dimension d = size();
                fld.move((d.width/2) - 200, (d.height/2) - (250 - 100));
                ((Window)fld).show();
                fld.dispose();
                int answer = fld.getSelectedIndex();
                if (answer != -1) {
                    gridCanvas.setFontString(fonts[answer].font);
                }
            }
        } else {
            processed = false;
        }
        return(processed);
    }
}
